<?php

	$tests = array(
		'1K'   => 'logs/test.1K.log',
		'10K'  => 'logs/test.10K.log',
		'100K' => 'logs/test.100K.log',
		'1M'   => 'logs/test.1M.log',
		'10M'  => 'logs/test.10M.log',
		'100M' => 'logs/test.100M.log'
	);

	function tailCustom($filepath, $lines = 1, $adaptive = true) {

		// Open file
		$f = @fopen($filepath, "rb");
		if ($f === false) return false;

		// Sets buffer size
		if (!$adaptive) $buffer = 4096;
		else $buffer = ($lines < 2 ? 64 : ($lines < 10 ? 512 : 4096));

		// Jump to last character
		fseek($f, -1, SEEK_END);

		// Read it and adjust line number if necessary
		// (Otherwise the result would be wrong if file doesn't end with a blank line)
		if (fread($f, 1) != "\n") $lines -= 1;
		
		// Start reading
		$output = '';
		$chunk = '';

		// While we would like more
		while (ftell($f) > 0 && $lines >= 0) {

			// Figure out how far back we should jump
			$seek = min(ftell($f), $buffer);

			// Do the jump (backwards, relative to where we are)
			fseek($f, -$seek, SEEK_CUR);

			// Read a chunk and prepend it to our output
			$output = ($chunk = fread($f, $seek)) . $output;

			// Jump back to where we started reading
			fseek($f, -mb_strlen($chunk, '8bit'), SEEK_CUR);

			// Decrease our line counter
			$lines -= substr_count($chunk, "\n");

		}

		// While we have too many lines
		// (Because of buffer size we might have read too many)
		while ($lines++ < 0) {

			// Find first newline and remove all text before that
			$output = substr($output, strpos($output, "\n") + 1);

		}

		// Close file and return
		fclose($f);
		return trim($output);

	}

	function tailSingleByte($file, $lines = 1) {
		$handle = fopen($file, "r");
		$linecounter = $lines;
		$pos = -2;
		$beginning = false;
		$text = array();
		while ($linecounter > 0) {
			$t = " ";
			while ($t != "\n") {
				if (fseek($handle, $pos, SEEK_END) == -1) {
					$beginning = true; 
					break; 
				}
				$t = fgetc($handle);
				$pos --;
			}
			$linecounter --;
			if ($beginning) rewind($handle);
			$text[$lines-$linecounter-1] = fgets($handle);
			if ($beginning) break;
		}
		fclose ($handle);
		return trim(implode("", array_reverse($text))); // array_reverse is optional: you can also just return the $text array which consists of the file's lines.
	}

	function tailFile($filepath, $lines = 1) {
		return trim(implode("", array_slice(file($filepath), -$lines)));
	}

	function tailShell($filepath, $lines = 1) {
		ob_start();
		passthru('tail -'  . $lines . ' ' . escapeshellarg($filepath));
		return trim(ob_get_clean());
	}

	function display($string) {
		echo nl2br(str_replace("\n", "#N\n", $string));
	}

	function measureStart() {
		return microtime(true);
	}

	function measureEnd($m) {
		return number_format(microtime(true) - $m, 10);
	}

	foreach ($tests as $k => $test) {
		$l = 1;
		while ($l <= 1000) {

			for ($j = 0; $j < 10; $j++) {

				$m = measureStart();
				tailCustom($test, $l);
				$m = measureEnd($m);
				echo implode(',', array('custom', $k, $l, $m)) . "\n";

				$m = measureStart();
				tailCustom($test, $l, false);
				$m = measureEnd($m);
				echo implode(',', array('customfb', $k, $l, $m)) . "\n";
				
				$m = measureStart();
				tailSingleByte($test, $l);
				$m = measureEnd($m);
				echo implode(',', array('singlebyte', $k, $l, $m)) . "\n";
				
				if ($k != '100M') {
					$m = measureStart();
					tailFile($test, $l);
					$m = measureEnd($m);
					echo implode(',', array('file', $k, $l, $m)) . "\n";
				}

				$m = measureStart();
				tailShell($test, $l);
				$m = measureEnd($m);
				echo implode(',', array('shell', $k, $l, $m)) . "\n";

			}

			if ($l >= 100) $l += 100;
			elseif ($l >= 10) $l += 10;
			else $l++;

		}
	}

?>
